module Robot
  module Simulator
    module Direction
      DIRECTIONS = [EAST = 0, NORTH = 1, WEST = 2, SOUTH = 3]
      TO_STRS = { EAST => 'EAST', NORTH => 'NORTH', WEST => 'WEST', SOUTH => 'SOUTH' }
      TO_CONSTANTS = TO_STRS.invert

      # Direction on the left when facing current direction
      # @return [Direction]
      def self.left(direction)
        validate direction

        DIRECTIONS[(direction + 1) % 4]
      end

      # Direction on the right when facing current direction
      # @return [Direction]
      def self.right(direction)
        validate direction

        DIRECTIONS[(direction - 1 + 4) % 4]
      end

      def self.to_str(direction)
        validate direction

        TO_STRS[direction]
      end

      def self.from_str(direction_str)
        key = (direction_str || "").upcase
        raise(ArgumentError, "Direction string '#{direction_str}' must be one of #{TO_CONSTANTS.keys}") unless TO_CONSTANTS.keys.include? key
        TO_CONSTANTS[key]
      end

      def self.validate(direction)
        raise(ArgumentError, "Direction must be one of #{DIRECTIONS}") unless DIRECTIONS.include?(direction)
      end

      private_class_method :validate
    end
  end
end
