module Robot
  module Simulator
    # Robot class corresponds to robot which can move around on the table.
    class Robot
      attr_reader :current_track, :table

      def initialize(table)
        @table = table
      end

      def place(coordinate, direction)
        return false unless is_valid_coordinate?(coordinate)

        self.current_track = Track.new coordinate, direction
      end

      def move
        return false unless is_on_table?

        newX = current_track.coordinate.x
        newX += 1 if current_track.facing == Direction::EAST
        newX -= 1 if current_track.facing == Direction::WEST

        newY = current_track.coordinate.y
        newY += 1 if current_track.facing == Direction::NORTH
        newY -= 1 if current_track.facing == Direction::SOUTH

        new_coordinate = Coordinate.new(newX, newY)
        return false unless is_valid_coordinate? new_coordinate

        self.current_track = Track.new new_coordinate, current_track.facing
      end

      def turn_left
        return false unless is_on_table?

        self.current_track = Track.new current_track.coordinate, Direction.left(current_track.facing)
      end

      def turn_right
        return false unless is_on_table?

        self.current_track = Track.new current_track.coordinate, Direction.right(current_track.facing)
      end

    private
      def current_track=(current_track)
        @current_track = current_track
      end

      def is_valid_coordinate?(coordinate)
        table.is_valid_coordinate? coordinate
      end

      def is_on_table?
        self.current_track != nil
      end
    end
  end
end
