require "thor"
require 'robot/simulator'

module Robot
  module Simulator
    module UI
      class CLI < Thor

        desc "start_game", "Start the robot simulator game"
        method_option :file, :aliases => "-f", :desc => "path to a text file containing robot commands"
        def start_game
          @controller = Controller.new

          if options[:file]
            commands = read_commands_from_file options[:file]
          else
            commands = read_commands_from_console
          end

          commands.each {|command| dispatch_command command }
        end

        default_task :start_game

      private
        def read_commands_from_console
          say "Please input robot commands, end with a blank line ...", :green
          commands = []
          loop do
            command = ask('>').strip
            break if command.empty?
            commands << command
          end
          commands
        end

        def read_commands_from_file(file_name)
          say "Reading robot commands from file #{file_name} ...", :green
          commands = []
          File.foreach(file_name) do |line|
            line = line.strip
            commands << line unless line.empty?
          end
          commands
        end

        def dispatch_command(command)
          action, param = command.split(/\s+/)
          say "Will execute command: #{command}", :cyan
          begin
            case action.upcase
            when 'PLACE'
              execute_place_command action, param
            when 'MOVE'
              @controller.move
            when 'LEFT'
              @controller.left
            when 'RIGHT'
              @controller.right
            when 'REPORT'
              report = @controller.report
              say report ? report.join(',') : 'Not on table'
            else
              error "Unknow command is ignored: #{command}"
            end
          rescue StandardError => err
            error err.message
          end
        end

        def execute_place_command(action, param)
          if param
            x, y, facing = param.split(/\s*\,\s*/)
            if x and y and facing
              @controller.place x.to_i, y.to_i, facing
            else
              error "Command PLACE needs 3 parameters in format of x,y,facing, got #{param}"
            end
          else
            error "Command PLACE is ignored because of lack parameters"
          end
        end
      end
    end
  end
end
